package com.geobioboo.twitterproj2;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Map.Entry;

public class ChiSquared {

	// returns top 10 chi squared
	public static void main(String[] args) throws IOException {
		BufferedReader friday = new BufferedReader(new FileReader("all-words-l5_100plus_fri.txt"));
		BufferedReader saturday = new BufferedReader(new FileReader("all_words_l5_scaled_100plus_sat.txt"));
		
		HashMap<String, Data> hash = new HashMap<String, Data>();
		
		String line;
		while (null != (line = friday.readLine()) && !"".equals(line)) { // insert friday into hash
			String[] wordAndCount = line.split("=");
			Data data = new Data();
			data.word = wordAndCount[0];
			data.friCount = Integer.valueOf(wordAndCount[1]);
			hash.put(wordAndCount[0], data);
		}
		while (null != (line = saturday.readLine()) && !"".equals(line)) {
			String[] wordAndCount = line.split("=");
			Data data = hash.get(wordAndCount[0]);
			if (data != null) {
				data.satCount = Integer.valueOf(wordAndCount[1]);
			}
		}
		
		ArrayList<Entry<String, Data>> sortedData = new ArrayList<Entry<String, Data>>(hash.entrySet());
		Collections.sort(sortedData, new Comparator<Entry<String, Data>>() {
			@Override
			public int compare(Entry<String, Data> arg0,
					Entry<String, Data> arg1) {
				return arg1.getValue().chiSquared() - arg0.getValue().chiSquared();
			}
        	
        });
		
		for (int i = 0; i < 10; i++) {
			System.out.println(sortedData.get(i));
		}
	}
}

class Data implements Comparable<Data>{
	// to be created, the friday count must be at least 100 
	// (if file contains words with >= 100 instances, as intended)
	String word;
	int friCount = 0;
	int satCount = 0;
	int chiSquared = -1;

	int chiSquared() {
		if (chiSquared == -1) {
			if (satCount == 0) { // this word was not in saturday
				chiSquared = 0;
			} else {
				chiSquared = (satCount - friCount) * (satCount - friCount) / friCount;
			}
		}
		return chiSquared;
		
	}

	@Override
	public int compareTo(Data o) {
		return chiSquared() - o.chiSquared();
	}
	
	public String toString() {
		return chiSquared + ", E=" + friCount + " O=" + satCount;
	}
}
