/*
 * Copyright 2007 Yusuke Yamamoto
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.geobioboo.twitterproj2;

import java.io.FileWriter;
import java.io.IOException;

import twitter4j.Status;
import twitter4j.StatusAdapter;
import twitter4j.StatusDeletionNotice;
import twitter4j.StatusListener;
import twitter4j.TwitterException;
import twitter4j.TwitterStream;
import twitter4j.TwitterStreamFactory;

/**
 * <p>This is a code example of Twitter4J Streaming API - sample method support.<br>
 * Usage: java twitter4j.examples.PrintSampleStream<br>
 * </p>
 *
 * @author Yusuke Yamamoto - yusuke at mac.com
 */
public final class PrintSampleStream extends StatusAdapter {
    /**
     * Main entry of this application.
     *
     * @param args
     */
    public static void main(String[] args) throws TwitterException {
        final TwitterStream twitterStream = new TwitterStreamFactory().getInstance();
        StatusListener listener = new StatusListener() {
            public void onStatus(Status status) {
                System.out.println("@" + status.getUser().getScreenName() + " - " + status.getText());
                write(status.getText() + "\n");
            }

            public void onDeletionNotice(StatusDeletionNotice statusDeletionNotice) {
                System.out.println("Got a status deletion notice id:" + statusDeletionNotice.getStatusId());
            }

            public void onTrackLimitationNotice(int numberOfLimitedStatuses) {
                System.out.println("Got track limitation notice:" + numberOfLimitedStatuses);
            }

            public void onScrubGeo(long userId, long upToStatusId) {
                System.out.println("Got scrub_geo event userId:" + userId + " upToStatusId:" + upToStatusId);
            }

            public void onException(Exception ex) {
                ex.printStackTrace();
            }
        };
        twitterStream.addListener(listener);
        twitterStream.sample();
        
        Thread timer = new Thread(new Runnable() {
			
			@Override
			public void run() {
				try {
					int SEC_TO_RUN = 30 * 60;
					Thread.sleep(SEC_TO_RUN * 1000);
					twitterStream.shutdown();
					Thread.sleep(1000);
					System.out.println("Stopped the twitter stream");
					System.out.println("Output is in: C:\\Users\\Geobio\\Eclipse\\Twitter Proj2\\twitterStream.txt");
					endWrite(); // finishes writing the file
				} catch (InterruptedException e) {
					System.out.println("Interrupted while sleeping");
				}
			}
		});
        
        timer.start();
    }
    
    static FileWriter fw;
    
    private static void write(String s) {
    	try {
    		if (fw == null)
    			fw = new FileWriter("twitterStream.txt", false);
    		fw.write(s);
    	} catch (IOException e) {}
    }
    
    private static void endWrite() {
    	try {
    		fw.close();
    	} catch (IOException e) {}
    }
}
