package com.geobioboo.twitterproj2;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Map.Entry;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class WordCount {
	
	/**
	 * returns the count of each word in a hash table or arraylist
	 * each word is using java's regex for \w (which is alphanumeric)
	 * @ when used before a word is treated as a username, and # is accepted as tag
	 */
	public static void main(String[] args) throws Exception {
		wordCount = new HashMap<String, Integer>();
		BufferedReader br = new BufferedReader(new FileReader("twitterStreamFiltered.txt"));
		
		// @username, #hashtag, word, word_with_underscore. these are each considered its own word
		// other punctuation and non-english/alphanumeric characters (eg e w/ accent) are ignored
		// or technically they're split, so cundo is split as "cu" & "ndo"
		// this results in invalid results, but oh well
		Pattern p = Pattern.compile("@{0,1}#{0,1}\\w+");
		Matcher m;
		
        String line;
        while ((line = br.readLine()) != null) {
        	m = p.matcher(line);
        	while (m.find()) {
        		addWord(m.group());
        	}
        }
        
        ArrayList<Entry<String, Integer>> al = 
        		new ArrayList<Entry<String, Integer>>(wordCount.entrySet());
        
        // descending count and then lexico
        Collections.sort(al, new Comparator<Entry<String, Integer>>() {
			@Override
			public int compare(Entry<String, Integer> arg0,
					Entry<String, Integer> arg1) {
				int rtn = arg1.getValue() - arg0.getValue(); // value is count
				if (rtn != 0)
					return rtn;
				else
					return arg0.getKey().compareTo(arg1.getKey());
			}
        	
        });
       
        // print to console the top 100 most popular words
        final int HIGHEST_N = 100; // return 10 words with top occurrences
        for (int i = 0; i < HIGHEST_N; i++) {
        	System.out.println(al.get(i));
        }
        
        
        
        // write the arraylist into all-words.txt
        for (Entry e : al) {
        	write(e.toString() + "\n");
        }
        endWrite();
        System.out.println("done");
        
        printWordsFor1b();
	}
	
	private static void printWordsFor1b() {
		String[] words = {"the", "of", "lol", "omg", "berkeley", "stanford", 
				"twitter", "facebook", "happy", "sad", "obama", "romney"};
		for (String w : words) {
			Integer count = wordCount.get(w);
			if (count == null)
				count = 0;
			System.out.println(w + " = " + count);
		}
	}
	
	static FileWriter fw;
    
    private static void write(String s) {
    	try {
    		if (fw == null)
    			fw = new FileWriter("all-words.txt", false);
    		fw.write(s);
    	} catch (IOException e) {}
    }
    
    private static void endWrite() {
    	try {
    		fw.close();
    	} catch (IOException e) {}
    }

	static HashMap<String, Integer> wordCount;
	
	static void addWord(String word) {
		int MIN_WORD_LENGTH = 5;
		
		// if shorter than min word length, then just ignore this word altogether
		if (MIN_WORD_LENGTH > 0)
			if (word.length() < MIN_WORD_LENGTH)
				return;
		
		String wordLowercase = word.toLowerCase();
		Integer previousCount = wordCount.get(wordLowercase);
		wordCount.put(wordLowercase, previousCount == null ? 1 : previousCount + 1);
	}
}
